import { ActionType, SemanticsType, } from "@sfroment/object";
export var MapConflictResolution;
(function (MapConflictResolution) {
    MapConflictResolution[MapConflictResolution["SetWins"] = 0] = "SetWins";
    MapConflictResolution[MapConflictResolution["DeleteWins"] = 1] = "DeleteWins";
})(MapConflictResolution || (MapConflictResolution = {}));
export class MapDRP {
    semanticsType = SemanticsType.pair;
    _conflictResolution;
    _map;
    constructor(conflictResolution) {
        this._map = new Map();
        this._conflictResolution = conflictResolution ?? MapConflictResolution.SetWins;
    }
    set(key, value) {
        this._map.set(key, value);
    }
    delete(key) {
        this._map.delete(key);
    }
    query_has(key) {
        return this._map.has(key);
    }
    query_get(key) {
        return this._map.get(key);
    }
    query_entries() {
        return Array.from(this._map.entries());
    }
    query_keys() {
        return Array.from(this._map.keys());
    }
    query_values() {
        return Array.from(this._map.values());
    }
    // simple hash function
    _computeHash(data) {
        let hash = 0;
        for (let i = 0; i < data.length; i++) {
            const char = data.charCodeAt(i);
            hash = (hash << 5) - hash + char;
            hash |= 0;
        }
        return hash.toString(16);
    }
    resolveConflicts(vertices) {
        if (!vertices[0].operation || !vertices[1].operation) {
            return { action: ActionType.Nop };
        }
        const values0 = vertices[0].operation.value;
        const values1 = vertices[1].operation.value;
        // if keys are different, return no-op
        if (values0[0] !== values1[0]) {
            return { action: ActionType.Nop };
        }
        // if both are delete operations, return no-op
        if (vertices[0].operation.opType === "delete" && vertices[1].operation.opType === "delete") {
            return { action: ActionType.Nop };
        }
        // if both are set operations, keep operation with higher hash value
        if (vertices[0].operation.opType === "set" && vertices[1].operation.opType === "set") {
            const hash0 = this._computeHash(JSON.stringify(values0[1]));
            const hash1 = this._computeHash(JSON.stringify(values1[1]));
            if (hash0 > hash1) {
                return { action: ActionType.DropRight };
            }
            if (hash0 < hash1) {
                return { action: ActionType.DropLeft };
            }
            // return no-op if two value are equal
            return { action: ActionType.Nop };
        }
        return this._conflictResolution === MapConflictResolution.SetWins
            ? {
                action: vertices[0].operation.opType === "set" ? ActionType.DropRight : ActionType.DropLeft,
            }
            : {
                action: vertices[0].operation.opType === "set" ? ActionType.DropLeft : ActionType.DropRight,
            };
    }
}
